@@ -60,7 +60,6 @@ void split_and_queue(queue_t *q, int maxrqsz, mblk_t *mp, rtp_header_t *rtp, int
 	}
 }
 
-
 void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_ts, struct sockaddr *addr, socklen_t addrlen)
 {
 	int i;
@@ -72,7 +71,14 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 	return_if_fail(mp!=NULL);
 	
 	msgsize=msgdsize(mp);
-	
+
+	if (msgsize<RTP_FIXED_HEADER_SIZE){
+		ortp_warning("Packet too small to be a rtp packet (%i)!",msgsize);
+		rtpstream->stats.bad++;
+		ortp_global_stats.bad++;
+		freemsg(mp);
+		return;
+	}
 	rtp=(rtp_header_t*)mp->b_rptr;
 	if (rtp->version!=2)
 	{
@@ -165,7 +171,7 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 		rtp_session_update_payload_type(session,rtp->paytype);
 	}
 	
-	if (!(session->flags & RTP_SESSION_RECV_SYNC)){
+	if (session->flags & RTP_SESSION_FIRST_PACKET_DELIVERED) {
 		int32_t slide=0;
 		int32_t safe_delay=0;
 		jitter_control_new_packet(&session->rtp.jittctl,rtp->timestamp,local_str_ts,&slide,&safe_delay);
@@ -179,8 +185,17 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 			rtp_signal_table_emit2(&session->on_timestamp_jump,(long)&rtp->timestamp);
 		}
 		else if (RTP_TIMESTAMP_IS_STRICTLY_NEWER_THAN(session->rtp.rcv_last_ts,rtp->timestamp)){
-			/* avoid very old packet to enqueued, because the user is no more supposed to get them */
-			ortp_debug("rtp_parse: silently discarding very old packet (ts=%i)",rtp->timestamp);
+			/* don't queue packets older than the last returned packet to the application*/
+			/* Call timstamp jumb in case of
+			 * large negative Ts jump or if ts is set to 0
+			*/
+			
+			if ( RTP_TIMESTAMP_IS_STRICTLY_NEWER_THAN(session->rtp.rcv_last_ts, rtp->timestamp + session->rtp.ts_jump) ){
+				ortp_warning("rtp_parse: negative timestamp jump");
+				rtp_signal_table_emit2(&session->on_timestamp_jump,
+							(long)&rtp->timestamp);
+			}
+			ortp_debug("rtp_parse: discarding too old packet (ts=%i)",rtp->timestamp);
 			freemsg(mp);
 			stats->outoftime++;
 			ortp_global_stats.outoftime++;
